/*
 * ContorollerDefinition: コントローラ定義 
 */

package org.dbc_works.app.sbkeyhook;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.Properties;

import org.dbc_works.lib.ui.ActionDefinition;
import org.dbc_works.lib.ui.ActionManager;
import org.dbc_works.lib.ui.MenuItemDefinition;

import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class ControllerDefinition
{
	//
	// 定数
	//

	// 「編集」メニューのメニューバー上のインデックス
	public static final int MENU_EDIT = 1;

	//
	// コンストラクタ
	//
	ControllerDefinition()
	{
	}

	//
	// 操作
	//

	// コントローラを作成する
	Component[] buildMainController(
		final MainFrame mainFrame,		// アクションを受けとるメインフレーム
		final Frame frame, 			// メニューを作成するフレーム
		final ActionManager manager)	// アクションマネージャー
	{
		final Properties menuLabelProperties = LabelManager.getMenuLabelProperties();

		final Toolkit tk = Toolkit.getDefaultToolkit();

		// ツールバー
		final ActionDefinition[] actionDef = new ActionDefinition[] {
			new ActionDefinition(
				mainFrame.addAction,
				menuLabelProperties.getProperty(LabelManager.MENU_EDIT_ADD),
				tk.getImage(getClass().getResource("/cmd-add.png")),
				ActionDefinition.Style.NORMAL),
			new ActionDefinition(
				mainFrame.editAction,
				menuLabelProperties.getProperty(LabelManager.MENU_EDIT_EDIT),
				tk.getImage(getClass().getResource("/cmd-edit.png")),
				ActionDefinition.Style.NORMAL),
			new ActionDefinition(
				mainFrame.deleteAction,
				menuLabelProperties.getProperty(LabelManager.MENU_EDIT_DELETE),
				tk.getImage(getClass().getResource("/cmd-delete.png")),
				ActionDefinition.Style.NORMAL),
			new ActionDefinition(
				mainFrame.undoAction,
				menuLabelProperties.getProperty(LabelManager.MENU_EDIT_UNDO),
				tk.getImage(getClass().getResource("/cmd-undo.png")),
				ActionDefinition.Style.NORMAL),
			new ActionDefinition(
				mainFrame.redoAction,
				menuLabelProperties.getProperty(LabelManager.MENU_EDIT_REDO),
				tk.getImage(getClass().getResource("/cmd-redo.png")),
				ActionDefinition.Style.NORMAL),
		};

		// 「ファイル」メニュー
		final MenuItemDefinition[] fileMenuDef =
			new MenuItemDefinition[] {
				new MenuItemDefinition(
					new ActionDefinition(
						mainFrame.closeAction,
						menuLabelProperties.getProperty(
							LabelManager.MENU_FILE_EXIT),
						null,
						ActionDefinition.Style.NORMAL),
					KeyEvent.VK_X),
			};

		// 「編集」メニュー
		final MenuItemDefinition[] editMenuDef =
			new MenuItemDefinition[] {
				new MenuItemDefinition(actionDef[3], KeyEvent.VK_Z, KeyEvent.VK_Z),
				new MenuItemDefinition(actionDef[4], KeyEvent.VK_Y, KeyEvent.VK_Y),
				MenuItemDefinition.SEPARATOR,
				new MenuItemDefinition(actionDef[0], KeyEvent.VK_A, KeyEvent.VK_A),
				new MenuItemDefinition(actionDef[1], KeyEvent.VK_E, KeyEvent.VK_E),
				new MenuItemDefinition(actionDef[2], KeyEvent.VK_D, KeyEvent.VK_D),
				};

		manager.buildMenu(
			menuLabelProperties.getProperty(LabelManager.MENU_FILE),
			KeyEvent.VK_F,
			fileMenuDef,
			frame);
		manager.buildMenu(
			menuLabelProperties.getProperty(LabelManager.MENU_EDIT),
			KeyEvent.VK_E,
			editMenuDef,
			frame);

		return manager.createToolBarComponents(actionDef);
	}
}
