/*
 * EditAccountInfoCommand: アカウント情報編集抽象基本クラス  
 */

package org.dbc_works.app.sbkeyhook.cmd;

import org.dbc_works.lib.model.Command;

import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.model.KeyHookDocument;

abstract public class EditAccountInfoCommand extends Command
{
	//
	// フィールド(実装)
	//

	// 操作対象ドキュメント
	private final KeyHookDocument document;
	
	// 追加アカウント情報
	private final AccountInfo	addAccountInfo;

	// 削除アカウント情報
	private final AccountInfo	deleteAccountInfo;

	//
	// コンストラクタ
	//
	protected EditAccountInfoCommand(
		String commandName,			// コマンド名
		AccountInfo addAccount,		// 追加アカウント情報
		AccountInfo deleteAccount,	// 削除アカウント情報
		KeyHookDocument doc)		// 対象ドキュメント
	{
		super(commandName);
		
		if (doc == null) {
			throw new NullPointerException();
		}
		
		document = doc;
		addAccountInfo = addAccount;
		deleteAccountInfo = deleteAccount;
	}
	
	//
	// 状態
	//
	
	// 編集コマンドかを返す
	public final boolean isEditCommand()
	{
		return (addAccountInfo != null && deleteAccountInfo != null);
	}
	
	// 追加アカウント情報の名前を返す
	public final String getAddAccountInfoName()
	{
		return (addAccountInfo != null ? addAccountInfo.name : "");
	}
	
	//
	// 再定義メソッド(状態)
	//

	// 実行可能かを返す
	public final boolean canExecute()
	{
		// 常に実行可能
		return true;
	}

	//
	// 再定義メソッド(操作)
	//

	// 実行する	
	public final Command.Result execCommand()
		throws Exception
	{
		if (deleteAccountInfo != null) {
			document.remove(deleteAccountInfo);
		} 
		if (addAccountInfo != null) {
			document.add(addAccountInfo);
		}
		return Result.SUCCESS;
	}

	// 実行を取り消す
	public final Command.Result cancelCommand()
		throws Exception
	{
		if (addAccountInfo != null) {
			document.remove(addAccountInfo); 
		}
		if (deleteAccountInfo != null) {
			document.add(deleteAccountInfo);
		}
		return Result.SUCCESS;
	}
}
