/*
 * AccountInfo: アカウント情報
 * 注記: 不変オブジェクト 
 */

package org.dbc_works.app.sbkeyhook.model;

import java.io.IOException;
import java.io.Writer;

import org.dbc_works.lib.environment.EnvInfo;

public final class AccountInfo
{
	//
	// 定数
	//
	
	// シリアライズ時の各項目区切り文字
	private static final String SEPARATOR = "\t";
	
	//
	// クラスメソッド(公開)
	//

	// 文字列を解析する	
	static AccountInfo parse(
		String formatted)	// 解析対象文字列
	{
		if (formatted == null) {
			throw new NullPointerException();
		}

		final String[] columns = EnvInfo.getInstance().split(SEPARATOR, formatted);
		if (columns.length < 3) {
			throw new IllegalArgumentException();
		}
		
		final String memo = columns.length == 4 ? unescape(columns[3]) : "";
		return new AccountInfo(columns[0], columns[1], columns[2], memo);
	}
	
	//
	// クラスメソッド(実装)
	//

	// 文字列をエスケープする
	private static String escape(
		String str)	// エスケープ対象文字列
	{
		if (str == null) {
			throw new NullPointerException();
		}

		final StringBuffer sb = new StringBuffer();

		// GUIとしてSwingを利用していると改行はWindowsでも'\n'のみとなる。
		final String separator = (0 <= str.indexOf(EnvInfo.NEWLINE) ? EnvInfo.NEWLINE : "\n");
		final String[] lines = EnvInfo.getInstance().split(separator, str);
		for (int line = 0; line < lines.length; line++) {
			if (0 < line) {
				sb.append("\\n");
			}
			for (int index = 0; index < lines[line].length(); index++) {
				switch (lines[line].charAt(index)) {
					case '\r':
						sb.append("\\r");
						break;
					case '\n':
						sb.append("\\n");
						break;
					case '\\':
						sb.append("\\\\");
						break;
					default:
						sb.append(lines[line].charAt(index));
				}
			}
		}
		
		return sb.toString();
	}
	
	// エスケープを解除する
	private static String unescape(
		String escaped)	// エスケープ文字列
	{
		if (escaped == null) {
			throw new NullPointerException();
		}
		
		final StringBuffer sb = new StringBuffer();
		int index = 0;
		while (index < escaped.length()) {
			final char c = escaped.charAt(index++); 
			if (c == '\\' && index < escaped.length()) {
				final char nextChar = escaped.charAt(index++);
				switch (nextChar) {
					case 'r':
						sb.append("\r");
						break;
					case 'n':
						sb.append("\n");
						break;
					case '\\':
						sb.append("\\");
						break;
					default:
						sb.append(c);
						sb.append(nextChar);
						break;
				}
			}
			else {
				sb.append(c);
			}
		}
		
		final String[] lines = EnvInfo.getInstance().split("\n", sb.toString());
		sb.setLength(0);
		for (int line = 0; line < lines.length; line++) {
			if (0 < line) {
				sb.append(EnvInfo.NEWLINE);
			}
			sb.append(lines[line]);
		}
		return sb.toString();
	}
	
	//
	// フィールド(公開)
	//
	
	public final String name;
	public final String loginId;
	public final String password;
	public final String memo;

	//
	// コンストラクタ
	//	
	public AccountInfo(
		String n,		// 名前
		String id,		// ログインID
		String pwd,		// パスワード
		String m)		// メモ
	{
		if (n == null) {
			throw new NullPointerException();
		}
		else if (n.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (id == null) {
			throw new NullPointerException();
		}
		else if (id.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (pwd == null) {
			throw new NullPointerException();
		}
		else if (pwd.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (m == null) {
			throw new NullPointerException();
		}

		name = n;
		loginId = id;
		password = pwd;
		memo = m;
	}

	//
	// 操作
	//
		
	// ストリームに書きこむ
	void serializeTo(
		Writer writer)	// 出力対象ストリームライタ
		throws IOException
	{
		if (writer == null) {
			throw new NullPointerException();
		}
		
		writer.write(name);
		writer.write(SEPARATOR);
		writer.write(loginId);
		writer.write(SEPARATOR);
		writer.write(password);
		writer.write(SEPARATOR);
		writer.write(escape(memo));
	}
}
