/*
 * Encryptor: 暗号化 / 復号
 */
package org.dbc_works.app.sbkeyhook.model;

import org.bouncycastle.crypto.BufferedBlockCipher;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.paddings.PaddedBufferedBlockCipher;
import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.digests.SHA256Digest;

final class Encryptor
{
	//
	// フィールド(実装)
	//
    
	private final BufferedBlockCipher cipher;
	private final KeyParameter keyParameter;
    
    //
    // コンストラクタ
    //
	Encryptor(
		byte[] key)	// 暗号化キー
	{
		if (key == null) {
			throw new NullPointerException();
		}
		else if (key.length == 0) {
			throw new IllegalArgumentException();
		}
		
		cipher = new PaddedBufferedBlockCipher(new CBCBlockCipher(new AESEngine()));
		
		final Digest digest = new SHA256Digest();
		final byte[] hash = new byte[digest.getDigestSize()]; 
		digest.reset();
		digest.update(key, 0, key.length);
		digest.doFinal(hash, 0);
		keyParameter = new KeyParameter(hash);
	}
    
    //
    // 操作
    //
    
    // 暗号化する
	byte[] encrypt(
		byte[] data)	// 暗号化対象データのバイト列
		throws CryptoException
	{
		if (data == null) {
			throw new NullPointerException();
		}

		if (data.length == 0) {
			return new byte[0];
		}
        
		cipher.init(true, keyParameter);
		return callCipher(data);
	}

	// 復号する    
	byte[] decrypt(
		byte[] encrypted)	// 暗号化データ
		throws CryptoException
	{
		if (encrypted == null) {
			throw new NullPointerException();
		}
		if (encrypted.length == 0) {
			return new byte[0];
		}
        
		cipher.init(false, keyParameter);
		return callCipher(encrypted);
	}
    
	//
	// 実装
	//
    
    // 暗号化 / 復号処理を行う
	private byte[] callCipher(
		byte[] data)	// 対象データ
		throws CryptoException
	{
		int size = cipher.getOutputSize(data.length);
		byte[] result = new byte[size];
		int outputLen = cipher.processBytes(data, 0, data.length, result, 0);
		outputLen += cipher.doFinal(result, outputLen);
        
		if (outputLen < size) {
			byte[] tmp = new byte[outputLen];
			System.arraycopy(result, 0, tmp, 0, outputLen);
			result = tmp;
		}
        
		return result;
	}
}
