/*
 * EditAccountInfoDialog: アカウント情報編集ダイアログ(AWT版)
 */
package org.dbc_works.app.sbkeyhook.view.awt;

import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Label;
import java.awt.TextField;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.awt.DialogUtilities;
import org.dbc_works.lib.ui.awt.MessageDialog;

import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class EditAccountInfoDialog extends Dialog
	implements ActionListener
{
	//
	// 定数
	//

	public static final int FILED_COLUMN = 40;
	public static final int FILED_ROW = 5;

	//
	// クラスメソッド(公開)
	//
	
	// 列制約を作成する
	public static GridBagConstraints[] createColumnConstraints()
	{
		final GridBagConstraints leftColConst = new GridBagConstraints();
		leftColConst.anchor = GridBagConstraints.NORTHWEST;
		leftColConst.fill = GridBagConstraints.NONE;
		leftColConst.gridx = 0;
		leftColConst.gridy = 0;
		leftColConst.gridwidth = 1;
		leftColConst.gridheight = 1;
		leftColConst.weightx = 0.0;
		leftColConst.weighty = 0.0;
			
		final GridBagConstraints rightColConst = new GridBagConstraints();
		rightColConst.anchor = GridBagConstraints.NORTHWEST;
		rightColConst.fill = GridBagConstraints.NONE;
		rightColConst.gridx = 1;
		rightColConst.gridy = 0;
		rightColConst.gridwidth = 1;
		rightColConst.gridheight = 1;
		rightColConst.weightx = 0.0;
		rightColConst.weighty = 0.0;
		
		return new GridBagConstraints[] {
			leftColConst,
			rightColConst,
		};
	}
	
	//
	// フィールド
	//

	private AccountInfo newAccountInfo;
	private final String[] accountNames;
	private boolean edited;

	private TextField accountName;
	private TextField loginId;
	private TextField password;
	private TextArea memo;
	private Button okButton;
	private Button cancelButton;

	//
	// コンストラクタ
	// 
	EditAccountInfoDialog(
		Frame owner,	// 親となるフレーム
		String title,	// ダイアログタイトル
		String[] names)	// 既存のアカウント情報名 
	{
		super(owner, title, true);

		if (names == null) {
			throw new NullPointerException();
		}
		
		newAccountInfo = null;
		accountNames = names;

		init();
		DialogUtilities.moveWindowToCenter(owner, this);
	}
	EditAccountInfoDialog(
		Frame owner,			// 親となるフレーム
		String title,			// ダイアログタイトル
		AccountInfo account)	// 編集アカウント情報(nullのとき追加)
	{
		super(owner, title, true);

		if (account == null) {
			throw new NullPointerException();
		}
		
		newAccountInfo = account;
		accountNames = null;

		init();
		DialogUtilities.moveWindowToCenter(owner, this);

		accountName.setText(account.name);
		loginId.setText(account.loginId);
		password.setText(account.password);
		memo.setText(account.memo);
		accountName.setEditable(false);
	}

	//
	// 状態
	//

	// アカウント情報を返す(キャンセルまたは未編集のときはnull) 
	AccountInfo getAccountInfo()
	{
		return (edited ? newAccountInfo : null);
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});

		accountName = new TextField(FILED_COLUMN);
		loginId = new TextField(FILED_COLUMN);
		password = new TextField(FILED_COLUMN); 
	
		final KeyAdapter updateStatusAdapter = new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
				edited = true;
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					newAccountInfo = null;
					close();
				}
			}
		}; 
		final TextField[] texts = new TextField[] {
			accountName,
			loginId,
			password,		 
		};
		for (int index = 0; index < texts.length; index++) {
			texts[index].addKeyListener(updateStatusAdapter);
		}
			
		memo = new TextArea(FILED_ROW, FILED_COLUMN);
		memo.addKeyListener(updateStatusAdapter);
			
		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[][] {
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_NAME)),
				accountName,
			},
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_ID)),
				loginId,
			},
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_PASSWORD)),
				password,
			},
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_MEMO)),
				memo,
			},
		};
	
		final GridBagConstraints[] constraints = org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.createColumnConstraints();
		add(GridBagUtilities.createSimpleTableStyleContainer(constraints, components),
			BorderLayout.CENTER);

		// 下部		
		okButton = DialogUtilities.createOkButton(this);
		cancelButton = DialogUtilities.createCancelButton(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		Button[] buttons = new Button[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < accountName.getText().length()
							&& 0 < loginId.getText().length()
							&& 0 < password.getText().length());
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		if (edited) {
			if (e.getSource() == okButton) {
				if (accountNames != null) {
					for (int index = 0; index < accountNames.length; index++) {
						if (accountNames[index].equals(accountName.getText())) {
							MessageDialog.showReportMessage(this, LabelManager.getLabelProperties().getProperty(LabelManager.MSG_NAMEALREADYEXIST));
							return; 
						}
					}
				}
				newAccountInfo = new AccountInfo(accountName.getText(),
												loginId.getText(),
												password.getText(),
												memo.getText());
			}
			else {
				newAccountInfo = null;
			}
		}
		else {
			newAccountInfo = null;
		}
		
		close();
	}
}
