/*
 * MainFrameImp: AWT版メインウィンドウ
 * 注記: BridgeパターンのImplementorクラス
 */
package org.dbc_works.app.sbkeyhook.view.awt;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Label;
import java.awt.List;
import java.awt.MenuBar;
import java.awt.TextField;
import java.awt.SystemColor;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.awt.MessageDialog;

import org.dbc_works.app.sbkeyhook.MainFrame;
import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class MainFrameImp extends Frame
	implements org.dbc_works.app.sbkeyhook.view.ImplementOperation
{
	//
	// 定数
	//

	// ツールバーインデックス
	private static final int INDEX_TOOLBAR_CONTAINER	= 0;

	//
	// クラスメソッド(公開)
	//
	
	// アカウント情報の列制約を作成する
	public static GridBagConstraints[] createAccountInfoColumnConstraints()
	{
		final GridBagConstraints leftColConst = new GridBagConstraints();
		leftColConst.anchor = GridBagConstraints.NORTHWEST;
		leftColConst.fill = GridBagConstraints.NONE;
		leftColConst.gridx = 0;
		leftColConst.gridy = 0;
		leftColConst.gridwidth = 1;
		leftColConst.gridheight = 1;
		leftColConst.weightx = 0.0;
		leftColConst.weighty = 0.0;
				
		final GridBagConstraints rightColConst = new GridBagConstraints();
		rightColConst.anchor = GridBagConstraints.NORTHEAST;
		rightColConst.fill = GridBagConstraints.HORIZONTAL;
		rightColConst.gridx = 1;
		rightColConst.gridy = 0;
		rightColConst.gridwidth = 1;
		rightColConst.gridheight = 1;
		rightColConst.weightx = 100.0;
		rightColConst.weighty = 0.0;
		
		return new GridBagConstraints[] {
			leftColConst,
			rightColConst,
		};
	}
	
	//
	// フィールド
	//
	
	private Container toolBar;

	private TextField selectedAccountId;
	private TextField selectedAccountPassword;
	
	private List	accountList;

	private Label	statusLabel;

	//
	// コンストラクタ
	// 注記: リフレクションで作成するため引数は指定できない 
	//
	public MainFrameImp()
	{
		super();
	}

	//
	// 実装
	//

	// メニューを初期化する
	private void initMenu()
	{
		final MenuBar menuBar = getMenuBar();
		final Font objMenuFont = org.dbc_works.lib.environment.EnvInfo.getInstance().getMenuFont(this);
		if (objMenuFont != null) {
			menuBar.setFont((Font)objMenuFont);
		}
	}

	// アカウント情報コンテナを作成する
	private Container createAccountInfoContainer()
	{
		selectedAccountId = new TextField();
		selectedAccountId.setEditable(false);
		
		selectedAccountPassword = new TextField();
		selectedAccountPassword.setEditable(false);
				
		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[][] {
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_ID)),
				selectedAccountId,
			},
			new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_PASSWORD)),
				selectedAccountPassword,
			},
		};
				
		final GridBagConstraints[] constraints = createAccountInfoColumnConstraints();
		return GridBagUtilities.createSimpleTableStyleContainer(constraints, components);
	}

	//
	// 再定義メソッド(ImplementOperationBase)
	//

	// コンポーネントを初期化する
	public void initComponent() 
	{
		// ウィンドウ背景色を設定する
		setBackground(SystemColor.menu);

		// メニューを初期化する
		initMenu();

		setLayout(new BorderLayout());
		
		// ツールバー作成
		toolBar = new Container();
		final FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
		layout.setHgap(0);
		toolBar.setLayout(layout);
		add(toolBar, BorderLayout.NORTH, INDEX_TOOLBAR_CONTAINER);

		// コントロール作成
		final Container accountContainer = new Container();
		accountContainer.setLayout(new BorderLayout());
		
		accountContainer.add(createAccountInfoContainer(), BorderLayout.NORTH);
		
		accountList = new List();
		accountList.setFont(getFont());
		
		accountContainer.add(accountList, BorderLayout.CENTER);

		add(accountContainer, BorderLayout.CENTER);

		statusLabel = new Label();
		add(statusLabel, BorderLayout.SOUTH);
		
		pack();
		
		// Fedora Core 4上のSun JRE 1.4.2_02で下記メソッドを呼びだすとマウスクリック
		// イベント発生時にアボートする
	//	entryList.requestFocus();
	}

	// メッセージを表示する
	public void showMessage(
		String msg)	// メッセージ文字列
	{
		MessageDialog.showReportMessage(this, msg);
	}

	//
	// 再定義メソッド(ToolBarOperation)
	//

	// ツールバー用コンポーネントを追加する
	public void addToolBarComponents(
		Component[] components)	// 設定コンポーネント
	{
		if (components == null) {
			throw new NullPointerException();
		}
		else if (components.length == 0) {
			throw new IllegalArgumentException();
		}

		final Container toolBar = (Container)getComponent(INDEX_TOOLBAR_CONTAINER);
		if (0 < toolBar.getComponentCount()) {
			toolBar.add(new Label(""));
		}
		for (int nIndex = 0; nIndex < components.length; nIndex++) {
			toolBar.add(components[nIndex]);
		}
	}

	//
	// 再定義メソッド(ImplementOperation)
	//

	// リスト項目選択変更処理インスタンスを設定する
	public void setListSelectionReceiver(
		final MainFrame.AbstractionReceiver receiver)	// リスト項目選択変更処理インスタンス
	{
		if (receiver == null) {
			throw new NullPointerException();
		}
		
		accountList.addItemListener(new ItemListener() {
			public void itemStateChanged(
				ItemEvent e)
			{
				receiver.receive();
			}
		});
	}

	// リスト項目編集処理インスタンスを設定する
	public void setEditAccountReceiver(
		final MainFrame.AbstractionReceiver receiver)	// リスト項目編集処理インスタンス
	{
		if (receiver == null) {
			throw new NullPointerException();
		}

		accountList.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e)
			{
				receiver.receive();
			}
		});
	}

	// 追加の状態を設定する
	public void setAdditionalState(
		Map properties)	// 状態プロパティ
	{
		if (properties == null) {
			throw new IllegalArgumentException();
		}
	}

	// 追加の状態を返す
	public Map getAdditionalState()
	{
		return new HashMap();
	}
	
	// 選択中かを返す
	public boolean hasSelection()
	{
		return 0 <= accountList.getSelectedIndex();
	}
	
	// 選択を返す
	public String getSelectedAccountName()
	{
		if (hasSelection() == false) {
			throw new IllegalStateException();
		}
		
		return accountList.getSelectedItem();
	}

	// アカウント情報名を設定する 
	public void setAccountInfoNames(
		String[] names)	// アカウント情報名の一覧
	{
		if (names == null) {
			throw new NullPointerException();
		}

		accountList.setVisible(false);
		accountList.removeAll();
		for (int index = 0; index < names.length; index++) {
			accountList.add(names[index]);
		}
		accountList.setVisible(true);
	}
	
	// アカウント情報名を設定する 
	public void selectAccount(
		AccountInfo account)	// アカウント情報
	{
		if (account != null) {
			for (int index = 0; index < accountList.getItemCount(); index++) {
				if (account.name.equals(accountList.getItem(index))) {
					accountList.select(index);
					accountList.makeVisible(index);
					
					selectedAccountId.setText(account.loginId);
					selectedAccountPassword.setText(account.password);
					break;
				}
			}
		}
		else {
			selectedAccountId.setText("");
			selectedAccountPassword.setText("");
		}
	}

	// パスワード入力ダイアログを表示する
	public String showPasswordDialog(
		String title,		// タイトル
		boolean confirm)	// 確認フラグ
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}

		final PasswordDialog dialog = new PasswordDialog(this, title, confirm);
		dialog.show();
		return dialog.getPassword();
	}

	// アカウント情報追加ダイアログを表示する
	public AccountInfo showAddAccountInfoDialog(
		String title,	// ダイアログタイトル
		String[] names)	// 既存のアカウント情報名の配列
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (names == null) {
			throw new NullPointerException();
		}

		final EditAccountInfoDialog dialog = new EditAccountInfoDialog(this, title, names);
		dialog.show();
		return dialog.getAccountInfo();
	}
	
	// アカウント情報編集ダイアログを表示する
	public AccountInfo showEditAccountInfoDialog(
		String title,				// ダイアログタイトル
		AccountInfo editAccount)	// 編集アカウント情報
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (editAccount == null) {
			throw new NullPointerException();
		}

		final EditAccountInfoDialog dialog = new EditAccountInfoDialog(this, title, editAccount);
		dialog.show();
		return dialog.getAccountInfo();
	}
 
	// 確認メッセージを表示する
	public boolean showConfirmMessage(
		String msg)	// メッセージ文字列 
	{
		if (msg == null) {
			throw new NullPointerException();
		}
		
		return MessageDialog.showConfirmMessage(this, msg); 
	}

	// ステータスメッセージを設定する
	public void setStatusMessage(
		String msg)	// 設定メッセージ
	{
		if (msg == null) {
			throw new NullPointerException();
		}
		
		statusLabel.setText(msg);
	}
}
