/*
 * PasswordDialog: パスワード入力ダイアログダイアログ(AWT版)
 */
package org.dbc_works.app.sbkeyhook.view.awt;

import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.awt.DialogUtilities;

import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class PasswordDialog extends Dialog
	implements ActionListener
{
	//
	// クラスメソッド(公開)
	//
	
	// 列制約を作成する
	public static GridBagConstraints[] createColumnConstraints()
	{
		final GridBagConstraints leftColConst = new GridBagConstraints();
		leftColConst.anchor = GridBagConstraints.NORTHWEST;
		leftColConst.fill = GridBagConstraints.NONE;
		leftColConst.gridx = 0;
		leftColConst.gridy = 0;
		leftColConst.gridwidth = 1;
		leftColConst.gridheight = 1;
		leftColConst.weightx = 0.0;
		leftColConst.weighty = 0.0;
				
		final GridBagConstraints rightColConst = new GridBagConstraints();
		rightColConst.anchor = GridBagConstraints.NORTHWEST;
		rightColConst.fill = GridBagConstraints.NONE;
		rightColConst.gridx = 1;
		rightColConst.gridy = 0;
		rightColConst.gridwidth = 1;
		rightColConst.gridheight = 1;
		rightColConst.weightx = 0.0;
		rightColConst.weighty = 0.0;
		
		return new GridBagConstraints[] {
			leftColConst,
			rightColConst,
		};
	}
	
	//
	// フィールド
	//

	public static final char CHAR_ECHO = '*';
	public static final int COLUMN_WIDTH = 30;
	private final boolean withConfirm;
	private boolean canceled;

	private TextField password;
	private TextField confirmPassword;
	private Button okButton;
	private Button cancelButton;

	//
	// コンストラクタ
	// 
	PasswordDialog(
		Frame owner,		// 親となるフレーム
		String title,		// ダイアログタイトル
		boolean confirm)	// 確認
	{
		super(owner, title, true);

		withConfirm = confirm;
		
		init();
		DialogUtilities.moveWindowToCenter(owner, this);
	}

	//
	// 状態
	//

	// アカウント情報を返す(キャンセルまたは未編集のときはnull) 
	String getPassword()
	{
		return (canceled == false ? password.getText() : null);
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});

		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[withConfirm ? 2 : 1][];
		password = new TextField(org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.COLUMN_WIDTH);
		components[0] = new Component[] {
			new Label(properties.getProperty(LabelManager.LBL_PASSWORD)),
			password,
		};
		if (withConfirm) {	
			confirmPassword = new TextField(org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.COLUMN_WIDTH);
			components[1] = new Component[] {
				new Label(properties.getProperty(LabelManager.LBL_CONFIRM)),
				confirmPassword,
			}; 
		}
		
		final KeyAdapter keyAdapter = new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
				switch (e.getKeyCode()) {
					case KeyEvent.VK_ENTER:
						if (okButton.isEnabled()) {
							canceled = false;
							close();
						}
						break;

					case KeyEvent.VK_ESCAPE:
						canceled = true;
						close();
						break;
				}
			}
		}; 
		final TextField[] texts = new TextField[] {
			password,		 
			confirmPassword,		 
		};
		for (int index = 0; index < texts.length; index++) {
			if (texts[index] != null) {
				texts[index].setEchoChar(CHAR_ECHO);
				texts[index].addKeyListener(keyAdapter);
			}
		}
	
		final GridBagConstraints[] constraints = createColumnConstraints();
		add(GridBagUtilities.createSimpleTableStyleContainer(constraints, components),
						BorderLayout.CENTER);

		// 下部		
		okButton = DialogUtilities.createOkButton(this);
		cancelButton = DialogUtilities.createCancelButton(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		final Button[] buttons = new Button[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < password.getText().length()
							&& (confirmPassword == null
								|| (0 < confirmPassword.getText().length()
									&& confirmPassword.getText().equals(password.getText()))));
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報
	{
		canceled = (e.getSource() == cancelButton);
		close();
	}
}
