/*
 * EditAccountInfoDialog: アカウント情報編集ダイアログ(Swing版)
 */
package org.dbc_works.app.sbkeyhook.view.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextArea;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.swing.DialogUtilities;

import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class EditAccountInfoDialog extends JDialog
	implements ActionListener
{
	//
	// フィールド
	//

	private AccountInfo newAccountInfo;
	private final String[] accountNames;
	private boolean edited;

	private JTextField accountName;
	private JTextField loginId;
	private JTextField password;
	private JTextArea memo;
	private JButton okButton;
	private JButton cancelButton;

	//
	// コンストラクタ
	// 
	EditAccountInfoDialog(
		JFrame owner,	// 親となるフレーム
		String title,	// ダイアログタイトル
		String[] names)	// 既存のアカウント情報名 
	{
		super(owner, title, true);

		if (names == null) {
			throw new NullPointerException();
		}
		
		newAccountInfo = null;
		accountNames = names;

		init();
		setLocationRelativeTo(owner);
	}
	EditAccountInfoDialog(
		JFrame owner,			// 親となるフレーム
		String title,			// ダイアログタイトル
		AccountInfo account)	// 編集アカウント情報(nullのとき追加)
	{
		super(owner, title, true);

		if (account == null) {
			throw new NullPointerException();
		}
		
		newAccountInfo = account;
		accountNames = null;

		init();
		setLocationRelativeTo(owner);

		accountName.setText(account.name);
		loginId.setText(account.loginId);
		password.setText(account.password);
		memo.setText(account.memo);
		accountName.setEditable(false);
	}

	//
	// 状態
	//

	// アカウント情報を返す(キャンセルまたは未編集のときはnull) 
	AccountInfo getAccountInfo()
	{
		return (edited ? newAccountInfo : null);
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});

		// コントロールを作成する
		accountName = new JTextField(org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.FILED_COLUMN);
		loginId = new JTextField(org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.FILED_COLUMN);
		password = new JTextField(org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.FILED_COLUMN); 

		final KeyAdapter updateStatusAdapter = new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
				edited = true;
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					newAccountInfo = null;
					close();
				}
			}
		};
		final JTextField[] texts = new JTextField[] {
			accountName,
			loginId,
			password,		 
		};
		for (int index = 0; index < texts.length; index++) {
			texts[index].addKeyListener(updateStatusAdapter);
		}
		
		memo = new JTextArea(org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.FILED_ROW, org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.FILED_COLUMN);
		memo.addKeyListener(updateStatusAdapter);
		
		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[][] {
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_NAME)),
				accountName,
			},
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_ID)),
				loginId,
			},
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_PASSWORD)),
				password,
			},
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_MEMO)),
				new JScrollPane(memo),
			},
		};

		final Container contentPane = getContentPane();
		final GridBagConstraints[] constraints = org.dbc_works.app.sbkeyhook.view.awt.EditAccountInfoDialog.createColumnConstraints();
		contentPane.add(GridBagUtilities.createSimpleTableStyleContainer(constraints, components),
						BorderLayout.CENTER);

		// 下部		
		okButton = DialogUtilities.createOkButton(this); 
		getRootPane().setDefaultButton(okButton);
		cancelButton = DialogUtilities.createCancelButton(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		JButton[] buttons = new JButton[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		contentPane.add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < accountName.getText().length()
							&& 0 < loginId.getText().length()
							&& 0 < password.getText().length());
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		if (edited) {
			if (e.getSource() == okButton) {
				if (accountNames != null) {
					for (int index = 0; index < accountNames.length; index++) {
						if (accountNames[index].equals(accountName.getText())) {
							getToolkit().beep();
							JOptionPane.showMessageDialog(this, LabelManager.getLabelProperties().getProperty(LabelManager.MSG_NAMEALREADYEXIST), getTitle(), JOptionPane.INFORMATION_MESSAGE);
							return; 
						}
					}
				}
				newAccountInfo = new AccountInfo(accountName.getText(),
												loginId.getText(),
												password.getText(),
												memo.getText());
			}
			else {
				newAccountInfo = null;
			}
		}
		else {
			newAccountInfo = null;
		}
		
		close();
	}
}
