/*
 * MainFrameImp: Swing版メインウィンドウ
 * 注記: BridgeパターンのImplementorクラス
 */
package org.dbc_works.app.sbkeyhook.view.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Map;
import java.util.Properties;
import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.swing.ImplementerUtilities;

import org.dbc_works.app.sbkeyhook.MainFrame;
import org.dbc_works.app.sbkeyhook.model.AccountInfo;
import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class MainFrameImp extends JFrame
	implements org.dbc_works.app.sbkeyhook.view.ImplementOperation
{
	//
	// 定数
	//

	// ツールバーインデックス
	private static final int INDEX_TOOLBAR_CONTAINER	= 0;

	//
	// フィールド
	//
	
	private final ImplementerUtilities utilities = new ImplementerUtilities();
	
	private JToolBar toolBar;
	
	private JTextField selectedAccountId;
	private JTextField selectedAccountPassword;
	
	private JList	accountList;
	
	private JLabel	statusLabel;

	//
	// コンストラクタ
	// 注記: リフレクションで作成するため引数は指定できない 
	//
	public MainFrameImp()
	{
		super();
	}

	//
	// 実装
	//

	// メニューを初期化する
	private void initMenu()
	{
		// 「表示」メニュー
		final Properties properties = LabelManager.getMenuLabelProperties();
		final JMenu menuView = new JMenu(properties.getProperty(LabelManager.MENU_VIEW));
		menuView.setMnemonic(KeyEvent.VK_V);
		if (utilities.setLookAndFeelSelectionMenuItems(this, menuView)) {
			final JMenuBar menuBar = getJMenuBar();
			final Font objMenuFont = org.dbc_works.lib.environment.EnvInfo.getInstance().getMenuFont(this);
			if (objMenuFont != null) {
				menuBar.setFont((Font)objMenuFont);
			}
			menuBar.add(menuView);
		}
	}

	// アカウント情報コンテナを作成する
	private Container createAccountInfoContainer()
	{
		selectedAccountId = new JTextField();
		selectedAccountId.setEditable(false);

		selectedAccountPassword = new JTextField();
		selectedAccountPassword.setEditable(false);
		
		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[][] {
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_ID)),
				selectedAccountId,
			},
			new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_PASSWORD)),
				selectedAccountPassword,
			},
		};
		
		final GridBagConstraints[] constraints = org.dbc_works.app.sbkeyhook.view.awt.MainFrameImp.createAccountInfoColumnConstraints();
		return GridBagUtilities.createSimpleTableStyleContainer(constraints, components);
	}
	
	//
	// 再定義メソッド(ImplementOperationBase)
	//

	// コンポーネントを初期化する
	public void initComponent() 
	{
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		// ウィンドウ背景色を設定する
		setBackground(SystemColor.menu);

		// メニューを初期化する
		initMenu();

		final Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		
		// ツールバー作成
		toolBar = new JToolBar();
		final FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
		layout.setHgap(0);
		toolBar.setLayout(layout);
		contentPane.add(toolBar, BorderLayout.NORTH, INDEX_TOOLBAR_CONTAINER);

		// コントロール作成
		final Container accountContainer = new Container();
		accountContainer.setLayout(new BorderLayout());

		accountContainer.add(createAccountInfoContainer(), BorderLayout.NORTH);
		
		accountList = new JList();
		accountList.setFont(getFont());
		accountContainer.add(new JScrollPane(accountList), BorderLayout.CENTER);

		contentPane.add(accountContainer, BorderLayout.CENTER);

		statusLabel = new JLabel();
		contentPane.add(statusLabel, BorderLayout.SOUTH);
		
		pack();
		
		// Fedora Core 4上のSun JRE 1.4.2_02で下記メソッドを呼びだすとマウスクリック
		// イベント発生時にアボートする
	//	entryList.requestFocus();
	}

	// メッセージを表示する
	public void showMessage(
		String msg)	// メッセージ文字列
	{
		getToolkit().beep();
		JOptionPane.showMessageDialog(this, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	//
	// 再定義メソッド(ToolBarOperation)
	//

	// ツールバー用コンポーネントを追加する
	public void addToolBarComponents(
		Component[] components)	// 設定コンポーネント
	{
		if (components == null) {
			throw new NullPointerException();
		}
		else if (components.length == 0) {
			throw new IllegalArgumentException();
		}

		if (0 < toolBar.getComponentCount()) {
			toolBar.add(new JLabel(""));
		}
		for (int nIndex = 0; nIndex < components.length; nIndex++) {
			toolBar.add(components[nIndex]);
		}
	}

	//
	// 再定義メソッド(ImplementOperation)
	//

	// リスト項目選択変更処理インスタンスを設定する
	public void setListSelectionReceiver(
		final MainFrame.AbstractionReceiver receiver)	// リスト項目選択変更処理インスタンス
	{
		if (receiver == null) {
			throw new NullPointerException();
		}
		
		accountList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(
				ListSelectionEvent e)
			{
				receiver.receive();
			}
		});
	}

	// リスト項目編集処理インスタンスを設定する
	public void setEditAccountReceiver(
		final MainFrame.AbstractionReceiver receiver)	// リスト項目編集処理インスタンス
	{
		if (receiver == null) {
			throw new NullPointerException();
		}

		accountList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(
				MouseEvent e) {
				if (2 <= e.getClickCount()) {
					if (0 <= accountList.locationToIndex(e.getPoint())) {
						receiver.receive();
					}
				}
			 }
		 });

		final String ENTER_ACTION = "EditAccount";
		accountList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), ENTER_ACTION);
		accountList.getActionMap().put(ENTER_ACTION, new AbstractAction() {
			public void actionPerformed(
				ActionEvent e)
			{
				receiver.receive();
			}
		});
	}

	// 追加の状態を設定する
	public void setAdditionalState(
		Map properties)	// 状態プロパティ
	{
		if (properties == null) {
			throw new IllegalArgumentException();
		}

	 	utilities.setAdditionalState(properties);
	}

	// 追加の状態を返す
	public Map getAdditionalState()
	{
	 	return utilities.getAdditionalState();
	}
	
	// 選択中かを返す
	public boolean hasSelection()
	{
		return accountList.isSelectionEmpty() == false;
	}
	
	// 選択を返す
	public String getSelectedAccountName()
	{
		if (hasSelection() == false) {
			throw new IllegalStateException();
		}
		
		return (String)accountList.getSelectedValue();
	}

	// アカウント情報名を設定する 
	public void setAccountInfoNames(
		String[] names)	// アカウント情報名の一覧
	{
		if (names == null) {
			throw new NullPointerException();
		}

		accountList.setListData(names);
	}
	
	// アカウント情報名を設定する 
	public void selectAccount(
		AccountInfo account)	// アカウント情報
	{
		if (account != null) {
			final javax.swing.ListModel model = accountList.getModel();
			for (int index = 0; index < model.getSize(); index++) {
				if (account.name.equals(model.getElementAt(index))) {
					accountList.setSelectedIndex(index);
					accountList.ensureIndexIsVisible(index);
					
					selectedAccountId.setText(account.loginId);
					selectedAccountPassword.setText(account.password);
					break;
				}
			}
		}
		else {
			selectedAccountId.setText("");
			selectedAccountPassword.setText("");
		}
	}

	// パスワード入力ダイアログを表示する
	public String showPasswordDialog(
		String title,		// タイトル
		boolean confirm)	// 確認フラグ
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}

		final PasswordDialog dialog = new PasswordDialog(this, title, confirm);
		dialog.show();
		return dialog.getPassword();
	}

	// アカウント情報追加ダイアログを表示する
	public AccountInfo showAddAccountInfoDialog(
		String title,	// ダイアログタイトル
		String[] names)	// 既存のアカウント情報名の配列
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (names == null) {
			throw new NullPointerException();
		}

		final EditAccountInfoDialog dialog = new EditAccountInfoDialog(this, title, names);
		dialog.show();
		return dialog.getAccountInfo();
	}
	
	// アカウント情報編集ダイアログを表示する
	public AccountInfo showEditAccountInfoDialog(
		String title,				// ダイアログタイトル
		AccountInfo editAccount)	// 編集アカウント情報
	{
		if (title == null) {
			throw new NullPointerException();
		}
		else if (title.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (editAccount == null) {
			throw new NullPointerException();
		}

		final EditAccountInfoDialog dialog = new EditAccountInfoDialog(this, title, editAccount);
		dialog.show();
		return dialog.getAccountInfo();
	}
 
	// 確認メッセージを表示する
	public boolean showConfirmMessage(
		String msg)	// メッセージ文字列 
	{
		if (msg == null) {
			throw new NullPointerException();
		}

		getToolkit().beep();
		return (JOptionPane.showConfirmDialog(this,
												msg,
												getTitle(),
												JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION);
	}

	// ステータスメッセージを設定する
	public void setStatusMessage(
		String msg)	// 設定メッセージ
	{
		if (msg == null) {
			throw new NullPointerException();
		}
	
		statusLabel.setText(msg);
		statusLabel.setToolTipText(msg);
	}
}
