/*
 * PasswordDialog: パスワード入力ダイアログダイアログ(Swing版)
 */
package org.dbc_works.app.sbkeyhook.view.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPasswordField;

import org.dbc_works.lib.ui.GridBagUtilities;
import org.dbc_works.lib.ui.swing.DialogUtilities;

import org.dbc_works.app.sbkeyhook.view.LabelManager;

public final class PasswordDialog extends JDialog
	implements ActionListener
{
	//
	// フィールド
	//

	private final boolean withConfirm;
	private boolean canceled;

	private JPasswordField password;
	private JPasswordField confirmPassword;
	private JButton okButton;
	private JButton cancelButton;

	//
	// コンストラクタ
	// 
	PasswordDialog(
		JFrame owner,		// 親となるフレーム
		String title,		// ダイアログタイトル
		boolean confirm)	// 確認
	{
		super(owner, title, true);

		withConfirm = confirm;
		
		init();
		setLocationRelativeTo(owner);
	}

	//
	// 状態
	//

	// アカウント情報を返す(キャンセルまたは未編集のときはnull) 
	String getPassword()
	{
		return (canceled == false ? String.valueOf(password.getPassword()) : null);
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});
	
		final Properties properties = LabelManager.getLabelProperties();
		final Component[][] components = new Component[withConfirm ? 2 : 1][];
		password = new JPasswordField(org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.COLUMN_WIDTH);
		components[0] = new Component[] {
			new JLabel(properties.getProperty(LabelManager.LBL_PASSWORD)),
			password,
		};
		if (withConfirm) {	
			confirmPassword = new JPasswordField(org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.COLUMN_WIDTH);
			components[1] = new Component[] {
				new JLabel(properties.getProperty(LabelManager.LBL_CONFIRM)),
				confirmPassword,
			}; 
		}
		
		final KeyAdapter keyAdapter = new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
				switch (e.getKeyCode()) {
					case KeyEvent.VK_ESCAPE:
						canceled = true;
						close();
						break;
				}
			}
		}; 
		final JPasswordField[] texts = new JPasswordField[] {
			password,		 
			confirmPassword,		 
		};
		for (int index = 0; index < texts.length; index++) {
			if (texts[index] != null) {
				texts[index].setEchoChar(org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.CHAR_ECHO);
				texts[index].addKeyListener(keyAdapter);
			}
		}

		final Container contentPane = getContentPane();
		final GridBagConstraints[] constraints = org.dbc_works.app.sbkeyhook.view.awt.PasswordDialog.createColumnConstraints();
		contentPane.add(GridBagUtilities.createSimpleTableStyleContainer(constraints, components),
						BorderLayout.CENTER);
	
		// 下部		
		okButton = DialogUtilities.createOkButton(this);
		getRootPane().setDefaultButton(okButton);
		cancelButton = DialogUtilities.createCancelButton(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		final JButton[] buttons = new JButton[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		contentPane.add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < password.getPassword().length
							&& (confirmPassword == null
								|| (0 < confirmPassword.getPassword().length
									&& String.valueOf(confirmPassword.getPassword()).equals(String.valueOf(password.getPassword())))));
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		canceled = (e.getSource() == cancelButton);
		close();
	}
}
