/*
 * AbstractSerializer: 
 */
package org.dbc_works.lib.model;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.dbc_works.lib.util.Utilities;

abstract public class AbstractSerializer
{
	//
	// クラスフィールド(実装)
	//
	
	private static final boolean DEBUG = false;
	
	//
	// クラスメソッド(実装)
	//
	
	// 情報を出力する
	private static void report(
		String msg)	// メッセージ
	{
		if (DEBUG) {
			System.err.println(msg);
		}
	}
	
	//
	// 操作
	//

	// シリアライズする
	public final void serialize(
		File serialize,	// シリアライズ対象
		File backup)	// バックアップファイル
		throws IOException, Exception
	{
		if (serialize == null) {
			throw new NullPointerException();
		}
		if (backup == null) {
			throw new NullPointerException();
		}
		
		if (DEBUG) {
			report("Serialize file path: " + serialize.getAbsolutePath());
			report("Backup file path: " + backup.getAbsolutePath());
		}
		
		if (backup.exists()) {
			if (backup.delete() == false) {
				if (DEBUG) {
					report("Fail: delete backup file.");
				}
				throw new IOException();
			}
		}
		
		final File temp = File.createTempFile("Serialize", null, serialize.getAbsoluteFile().getParentFile());
		if (DEBUG) {
			report("Temporary file path: " + temp.getAbsolutePath());
		}

		OutputStream stream = null;
		try {
			stream = new FileOutputStream(temp);
			doSerialize(stream);
		}
		finally {
			try {
				if (stream != null) {
					stream.close();	
				}
			}
			catch (Exception e) {
				// Do nothing.
				Utilities.reportException("", e);
			}
		}

		try {
			if (serialize.exists() && serialize.renameTo(backup) == false) {
				if (DEBUG) {
					report("Fail: rename current file to backup.");
				}
				throw new IOException("Fail: rename current file to backup.");
			}
		}
		catch (Exception e) {
			try {
				temp.delete();
			}
			catch (Exception exp) {
				// Do nothing.
				Utilities.reportException("", exp);
			}
			throw e;
		}
		try {
			if (temp.renameTo(serialize) == false) {
				if (DEBUG) {
					report("Fail: rename temporary file to serialize.");
				}
				throw new IOException("Fail: rename temporary file to serialize.");
			}
		}
		catch (Exception e) {
			try {
				backup.renameTo(serialize);
			}
			catch (Exception exp) {
				// Do nothing.
				Utilities.reportException("", exp);
			}
			throw e;
		}
	}
	
	//
	// 再定義必須メソッド(実装)
	//

	abstract protected void doSerialize(OutputStream stream) throws Exception;
}
