/*
 * ActionManager: Swing用アクションマネージャー 
 */
package org.dbc_works.lib.ui.swing;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.EventListener;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.dbc_works.lib.ui.ActionDefinition;
import org.dbc_works.lib.ui.MenuItemDefinition;

public final class ActionManager extends org.dbc_works.lib.ui.ActionManager
{
	//
	// コンストラクタ
	//
	public ActionManager()
	{
		super();
	}

	//
	// 再定義メソッド(操作)
	//

	// メニューを作成する 	
	public void buildMenu(
		String strLabel,					// 表記
		int nMenuMnemonic,					// ニーモニック
		MenuItemDefinition[] definitions,	// メニュー項目定義
		Frame frameTarget)					// 作成対象フレーム
	{
		if (DEBUG) {
			if (strLabel == null) {
				throw new NullPointerException();
			}
			else if (strLabel.length() == 0) {
				throw new IllegalArgumentException();
			}

			if (definitions == null) {
				throw new NullPointerException();
			}
			else if (definitions.length == 0) {
				throw new IllegalArgumentException();
			}

			if (frameTarget == null) {
				throw new NullPointerException();
			}
		}

		JMenuBar menuBar = ((JFrame)frameTarget).getJMenuBar(); 
		if (menuBar == null) {
			menuBar = new JMenuBar();
			((JFrame)frameTarget).setJMenuBar(menuBar);
		}

		final JMenu menu = new JMenu(strLabel);

		for (int nIndex = 0; nIndex < definitions.length; nIndex++) {
			final MenuItemDefinition itemDef = definitions[nIndex];
			if (itemDef == MenuItemDefinition.SEPARATOR) {
				menu.addSeparator();
			}
			else {
				final ActionDefinition actionDef = itemDef.action;
				JMenuItem item;
				if (actionDef.style == ActionDefinition.Style.CHECK) {
					JCheckBoxMenuItem checkItem = new JCheckBoxMenuItem(actionDef.label);
					checkItem.addItemListener((ItemListener)actionDef.listener);
					item = checkItem;
				}
				else if (actionDef.style == ActionDefinition.Style.NORMAL) {
					item = new JMenuItem(actionDef.label);
					item.addActionListener((ActionListener)actionDef.listener);
				}
				else {
					throw new IllegalArgumentException();
				}

				item.setMnemonic(itemDef.mnemonic);
				if (itemDef.accelerator != KeyEvent.CHAR_UNDEFINED) {
					item.setAccelerator(KeyStroke.getKeyStroke(itemDef.accelerator, ActionEvent.CTRL_MASK));
				}
				if (actionDef.image != null) {
					item.setIcon(new ImageIcon(actionDef.image));
				}

				menu.add(item);

				getMenuItemMap().put(actionDef.listener, item);
			}
		}

		menu.setMnemonic(nMenuMnemonic);
		menuBar.add(menu);
	}

	// 有効 / 無効を設定する
	public void setEnable(
		EventListener listener,	// リスナー
		boolean bEnable)		// 有効状態
	{
		if (DEBUG) {
			if (listener == null) {
				throw new NullPointerException();
			}
		}

		final Map mapMenuItem = getMenuItemMap(); 
		if (mapMenuItem.containsKey(listener)) {
			JMenuItem item = (JMenuItem)mapMenuItem.get(listener);
			item.setEnabled(bEnable);
		}

		final Map mapToolBarButton = getButtonMap(); 
		if (mapToolBarButton.containsKey(listener)) {
			Component component = (Component)mapToolBarButton.get(listener);
			component.setEnabled(bEnable);
		}
	}

	// チェック状態を設定する
	public void setCheckStatus(
		ItemListener listener,	// リスナー
		boolean bCheck)		// チェック状態
	{
		if (DEBUG) {
			if (listener == null) {
				throw new NullPointerException();
			}
		}

		final Map mapMenuItem = getMenuItemMap(); 
		if (mapMenuItem.containsKey(listener)) {
			JCheckBoxMenuItem checkItem = (JCheckBoxMenuItem)mapMenuItem.get(listener);
		//	checkItem.setState(bCheck);
			if (checkItem.getState() != bCheck) {
				checkItem.doClick();
			}
		}
	}
	
	//
	// 再定義メソッド(実装)
	//

	// ツールバー用コンポーネントを作成する
	protected Component createToolBarComponent(
		ActionDefinition actionDefinition)	// アクション定義
	{
		if (actionDefinition == null) {
			throw new NullPointerException();
		}
		else if (actionDefinition.style != ActionDefinition.Style.NORMAL) {
			// 未実装
			throw new IllegalArgumentException();
		}

		JButton button;
		if (actionDefinition.image != null) {
			button = new JButton(new ImageIcon(actionDefinition.image));
			button.setToolTipText(getStrictLabel(actionDefinition.label)); 
		}
		else {
			button = new JButton(getStrictLabel(actionDefinition.label));
		}
		button.addActionListener((ActionListener)actionDefinition.listener);
		getButtonMap().put(actionDefinition.listener, button);
		return button;
	}
}
